/*
 * GPI-Based Printing.  This file was created by James Scott Jr.
 * USDate 28 Feb 1997.
*/

/***************************************************************************/
/*                                                                         */
/* GPRINT - Graphics Print Program Library for PM.  Contains several      */
/*          routines that can be used in a final project to implement      */
/*          printing using PM's Advanced Printing Functions                */
/*                                                                         */
/***************************************************************************/


/*---------------*/
/* Include files */
/*---------------*/
#define  INCL_BASE
#define  INCL_DEV         /* Include OS/2 Device Interface */
#define  INCL_GPI
#define  INCL_WIN
#define  INCL_SPL         /* Include OS/2 PM Spooler Interface   */
#define  INCL_DOS
#define  INCL_DOSFILEMGR
#define  INCL_DOSMEMMGR
#define  INCL_DOSPROCESS
#define  INCL_DOSSESMGR
#define  INCL_SPLDOSPRINT
#define  INCL_DOSPROCESS
#define  INCL_WINWINDOWMGR       /* Window Manager Functions     */
#define  INCL_WINMENUS           /* Window Menu Functions        */
#define  INCL_WINSTDFILE
#define  INCL_WINACCELERATORS
#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include "GPRINT.h"


MRESULT APIENTRY fnClientWMCreate( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT APIENTRY fnClientCommand( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT APIENTRY fnHandlePopupMenu( HWND hwnd, ULONG idMenu );

/***************************************************************************/
/* Main - sets up Presentation Manager services; handles command-line      */
/* arguments.                                                              */
/***************************************************************************/
int main()

{
 HAB       hab;                                  /* Anchor block handle */
 HMQ       hmq;                                  /* Message queue handle */
 FRAMECDATA fcdata;
 QMSG      qmsg;                                 /* Message data structure */
 HWND      hwndClient, hwndFrame;                /* Frame and client window handles */
 WINDOWINFO pWD;                                 /* Window words */

  memset( (PVOID)&pWD, 0, sizeof(WINDOWINFO) );

/*------------------------------------------------------------------*/
/* Standard PM window creation, message loop, and termination.      */
/*------------------------------------------------------------------*/
 hab = WinInitialize (0);

 pWD.hab = hab;
 strcpy( pWD.szTitle, "CV2TAB Pkg: GPI Based Printing");

 hmq = WinCreateMsgQueue (hab, 0);

 WinRegisterClass(hab,
                  WIN_CLASS_NAME,
                  wpClientWndProc,
                  CS_SIZEREDRAW,
                  (2 * sizeof(ULONG))
                 );


 fcdata.cb            = sizeof(FRAMECDATA);
 fcdata.hmodResources = (HMODULE)0;
 fcdata.idResources   = IDW_FRAME_WINDOW;
 fcdata.flCreateFlags =    FCF_TITLEBAR      |   /* Window creation flags  */
                           FCF_SYSMENU       |
                           FCF_SIZEBORDER    |
                           FCF_MINMAX        |
                           FCF_SHELLPOSITION |
                           FCF_TASKLIST      |
                           FCF_MENU          |
                           FCF_ICON;

 hwndFrame = WinCreateWindow( HWND_DESKTOP,
                              WC_FRAME,
                              (PSZ)0,
                              0L,
                              0,0,
                              0,0,
                              0,
                              HWND_BOTTOM,
                              IDW_FRAME_WINDOW,
                              &fcdata,
                              0
                            );

 hwndClient = WinCreateWindow( hwndFrame,
                               WIN_CLASS_NAME,
                               pWD.szTitle,
                               WS_VISIBLE,
                               0,0,
                               0,0,
                               0,
                               HWND_TOP,
                               FID_CLIENT,
                               (PWINDOWINFO)&pWD,
                               0
                             );

 if( hwndClient == (HWND)0 )
   {
    WinDestroyMsgQueue (hmq);
    WinTerminate (hab);
    exit(EXIT_FAILURE);
   }

 WinSetWindowPos(hwndFrame, HWND_TOP, 0, 0, 0, 0, SWP_ACTIVATE | SWP_SHOW );

 while (WinGetMsg (hab, &qmsg, 0, 0, 0))        /* Start message loop */
    WinDispatchMsg (hab, &qmsg);

 WinDestroyWindow(hwndFrame);
 WinDestroyMsgQueue (hmq);
 WinTerminate (hab);

 return 0;
}


/***************************************************************************/
/* Client window procedure - kicks off a thread to load the user-requested */
/* file.  Also records writes any selected commands to the client area.    */
/* Add other processing as necessary.                                      */
/***************************************************************************/
MRESULT EXPENTRY wpClientWndProc (HWND hwnd, ULONG msg, MPARAM   mp1, MPARAM   mp2)
{
 PWINDOWINFO pWD = WinQueryWindowPtr(hwnd, QWL_USER);

 switch (msg)
 {
  case WM_BUTTON2DOWN:
  case WM_CONTEXTMENU:
       return fnHandlePopupMenu( hwnd, IDM_POPUP );

  case WM_COMMAND:
       return fnClientCommand( hwnd, msg, mp1, mp2);

  case WM_ERASEBACKGROUND:
       return (MRESULT)TRUE;

  case WM_CREATE:
       return fnClientWMCreate( hwnd, msg, mp1, mp2);

  case WM_PAINT:
       {
        HPS hps;
        RECTL rectClientWindow;
        hps = WinBeginPaint(hwnd, NULLHANDLE, &rectClientWindow);
        WinFillRect(hps, &rectClientWindow, CLR_PALEGRAY );
        WinQueryWindowRect( hwnd, &rectClientWindow);
        WinDrawText(hps,
                    sizeof("Press the right mouse button for an action menu."),
                    (PSZ)"Press the right mouse button for an action menu.",
                    &rectClientWindow, 0L, 0L, DT_VCENTER | DT_CENTER);
        WinEndPaint (hps);
       }
       break;

      /* This is the multi thread communication section                      */
      /* All threads post this message with their ID's and Handle on startup */
      /* and their ID and nullhandle on close                                */
      /* We control them by sending a IDF_CLOSE during our WM_CLOSE message  */
 case UM_INFORM:                     /* mp1.1=function, mp2=parm */
      switch(SHORT1FROMMP(mp1))
       {
        case IDF_CLOSE:                      /* Shutdown Processing */
          WinPostMsg(hwnd, WM_QUIT, 0L, 0L); /* --- see object.main */
          break;

        case IDF_PRINT:                     /* Done  Printing       */
          if(!SHORT1FROMMP(mp2) ) WinAlarm(HWND_DESKTOP, WA_NOTE);
           else WinAlarm(HWND_DESKTOP, WA_WARNING);
          break;

        case IDO_PRINTER:                   /* Start Printing       */
          if(pWD) pWD->hwndPrtObject = (HWND)mp2;     /* Save handle OR    */
          if( pWD->hwndPrtObject == (HWND)0 )         /* close application */
                WinPostMsg(hwnd, WM_CLOSE, 0L, 0L);
          break;

        default:
          break;
       }  /* end while UM_INFORM */
      return (MRESULT)0;

 case WM_CLOSE:
       if(pWD)
       if(pWD->hwndPrtObject)
          {
           WinPostMsg(pWD->hwndPrtObject, UM_INFORM, MPFROMSHORT(IDF_CLOSE), 0L);
           return (MRESULT)0;
          }
      break;

  case WM_DESTROY:
       WinDestroyWindow(pWD->hwndMenu);
       fnPrnDestroyPrinterList(&(pWD->prn));
       break;

  default:
        break;
 }   /* Endswitch msg */

 return (WinDefWindowProc(hwnd, msg, mp1, mp2) );
}   /* End of function */


/******************************************************************
* fnClientWMCreate - Processing for WM_CREATE
*/
MRESULT APIENTRY fnClientWMCreate( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
 PWINDOWINFO pWD ;

 pWD = (PWINDOWINFO)mp1;
 pWD->hwnd = hwnd;
 WinSetWindowPtr (hwnd, QWL_USER, pWD);

 /*Ŀ
  * Get List of print queues     
  *
 */
 if ( fnPrnCreatePrinterList( &(pWD->prn)) ) pWD->prn.hAB = pWD->hab;
     else {
           WinMessageBox( HWND_DESKTOP, HWND_DESKTOP,
              "Probable cause: No Defined Pinters.",
              "  System error: Get default print Queue Name.",
              0, MB_APPLMODAL | MB_CUACRITICAL | MB_CANCEL);

           WinPostMsg(hwnd, WM_QUIT, 0L, 0L);
           return (MRESULT)TRUE;  /* Cancel Statup - No Pointer */
          }

 /*Ŀ
  * Create Print Thread          
  *
 */
 if( fnCreatePrinterObject((PHWND)&(pWD->hwnd)) == -1 )
     {
      WinMessageBox( HWND_DESKTOP, HWND_DESKTOP,
                     "Probable cause: Unknown.",
                     " Fatal error: Create printer thread failed!",
                     0, MB_APPLMODAL | MB_CUACRITICAL | MB_CANCEL);

      WinPostMsg(hwnd, WM_QUIT, 0L, 0L);
      return (MRESULT)TRUE;  /* Cancel Statup - No Pointer */
     }

 return (WinDefWindowProc(hwnd, msg, mp1, mp2) );
}

/******************************************************************
* fnClientCommand - Processing for WM_COMMAND
*/
MRESULT APIENTRY fnClientCommand( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
 PWINDOWINFO pWD = WinQueryWindowPtr(hwnd, QWL_USER);

 switch (SHORT1FROMMP(mp1))
 {
  case IDM_FILE_ABOUT:
       fnProcessAboutDlg( WinQueryWindow(hwnd, QW_PARENT),
                          "About/PM:GPI Based Printing"
                        );
       break;
  case IDM_FILE_SETUP:
       fnProcessPrintDlg(  WinQueryWindow(hwnd, QW_PARENT) );
       break;

  case IDM_FILE_PRINT:
        fnPrintRequest( pWD, "CV2TAB Package Printing...");
       break;

  default:
      return (WinDefWindowProc(hwnd, msg, mp1, mp2) );
 }  /* end switch WM_COMMAND */

 return (MRESULT)FALSE;
}

/******************************************************************
* fnHandlePopupMenu - Processing for static popup menus
*/
MRESULT APIENTRY fnHandlePopupMenu( HWND hwnd, ULONG idMenu )
{
 POINTL      ptl;
 PWINDOWINFO pWD = WinQueryWindowPtr(hwnd, QWL_USER);

 if ( pWD == (PWINDOWINFO)0 )
     return (MRESULT)FALSE;

 if( pWD->hwndMenu == (HWND)0 )
     pWD->hwndMenu = WinLoadMenu( WinQueryWindow(hwnd, QW_PARENT),
                                  (HMODULE)0,
                                  idMenu
                                );

 if( pWD->hwndMenu == (HWND)0 )
     return (MRESULT)FALSE;

 WinQueryPointerPos(HWND_DESKTOP, &ptl);
 WinPopupMenu(HWND_DESKTOP,
              hwnd,
              pWD->hwndMenu,
              ptl.x,
              ptl.y,
              0,
              PU_NONE | PU_KEYBOARD |
              PU_MOUSEBUTTON1 | PU_MOUSEBUTTON2 |
              PU_HCONSTRAIN | PU_VCONSTRAIN
             );

 return (MRESULT)TRUE;
}

  /**************************************************************************/
  /* PRIVATE FUNCTION : Display message panel with About information.       */
  /**************************************************************************/
LONG APIENTRY fnProcessAboutDlg (HWND hwndFrame, PSZ pszAppName)
{
  return (LONG)WinDlgBox( HWND_DESKTOP,
                             hwndFrame,
                             wpAboutProc,
                             0L,
                             IDD_ABOUT,
                             pszAppName
                        );
} /* End of Function */

  /**************************************************************************/
  /* PRIVATE FUNCTION : DlgProc to handle custom about dialog.              */
  /**************************************************************************/
MRESULT EXPENTRY wpAboutProc (HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    ULONG   ulScrWidth, ulScrHeight;
    RECTL   Rectl;
    SWP     Swp;
    HPS     hps;

    switch (msg)
    {
        case WM_INITDLG:
            /* Center dialog on screen */
            ulScrWidth  = WinQuerySysValue (HWND_DESKTOP, SV_CXSCREEN);
            ulScrHeight = WinQuerySysValue (HWND_DESKTOP, SV_CYSCREEN);
            WinQueryWindowRect (hwnd, &Rectl);
            WinSetWindowPos (hwnd, HWND_TOP, (ulScrWidth-Rectl.xRight)/2,
                (ulScrHeight-Rectl.yTop)/2, 0, 0, SWP_MOVE | SWP_ACTIVATE);

            /* Set application title */
            WinSetDlgItemText (hwnd, 10001, (PSZ)mp2);  // next to Icon
            break;

        case WM_PAINT:
            hps = WinBeginPaint (hwnd,0,0);
            WinQueryWindowRect (hwnd, &Rectl);
            WinFillRect (hps, &Rectl, CLR_PALEGRAY);
            WinDrawBorder (hps, &Rectl,
                WinQuerySysValue(HWND_DESKTOP,SV_CXDLGFRAME),
                WinQuerySysValue(HWND_DESKTOP,SV_CYDLGFRAME),
                CLR_DARKGRAY, CLR_WHITE, DB_RAISED);

            GpiMove (hps, (PPOINTL)&Rectl);
            Rectl.xRight--;
            Rectl.yTop--;
            GpiBox (hps, DRO_OUTLINE, (PPOINTL)&Rectl.xRight, 0L, 0L);

            WinQueryWindowPos (WinWindowFromID (hwnd, 10002), &Swp);  // Top Box
            Rectl.xLeft   = Swp.x-1;
            Rectl.yBottom = Swp.y-1;
            Rectl.xRight  = Swp.x + Swp.cx + 1;
            Rectl.yTop    = Swp.y + Swp.cy + 1;
            WinDrawBorder (hps, &Rectl, 1L, 1L,
                CLR_DARKGRAY, CLR_WHITE, DB_DEPRESSED);

            WinQueryWindowPos (WinWindowFromID (hwnd, 10003), &Swp); // Bottom Box
            Rectl.xLeft   = Swp.x-1;
            Rectl.yBottom = Swp.y-1;
            Rectl.xRight  = Swp.x + Swp.cx + 1;
            Rectl.yTop    = Swp.y + Swp.cy + 1;
            WinDrawBorder (hps, &Rectl, 1L, 1L,
                CLR_DARKGRAY, CLR_WHITE, DB_DEPRESSED);

            WinEndPaint (hps);
            break;

        case WM_COMMAND:
            WinDismissDlg (hwnd, DID_OK);
            break;

        default:
            return (WinDefDlgProc (hwnd, msg, mp1, mp2));
    }


 return (MRESULT)0;
}


  /**************************************************************************/
  /* PRIVATE FUNCTION : Display Printer Destination Dialog                  */
  /**************************************************************************/
LONG APIENTRY fnProcessPrintDlg(HWND hwndFrame)
{
 PWINDOWINFO     pWD;

 pWD = (PWINDOWINFO)WinQueryWindowULong(
                        WinWindowFromID( hwndFrame, FID_CLIENT),
                                    QWL_USER);
 return (LONG)WinDlgBox( HWND_DESKTOP,
                         hwndFrame ,
                         wpPrintDlg,
                         0L,
                         IDD_PRINT,
                         pWD
                       );
} /* End of Function */

/* Ŀ
 *  Printer Destination Dialog                                          
 * 
*/
MRESULT EXPENTRY wpPrintDlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
 PWINDOWINFO        pWD = (PWINDOWINFO)WinQueryWindowULong(hwnd, QWL_USER);
 SHORT       NotifyCode = 0;

 switch(msg)
   {

    case WM_INITDLG:
         WinSetWindowULong( hwnd, QWL_USER, (ULONG)mp2);
         pWD = (PWINDOWINFO)mp2;
         WinEnableWindow(WinWindowFromID(hwnd, DID_SETUP), FALSE);
         WinEnableWindow(WinWindowFromID(hwnd, DID_FONTNAME), FALSE);
         WinPostMsg(hwnd, WM_USER, 0L, 0L);
         return (MRESULT)TRUE;

    case WM_USER:
         fnPrintListBox( &(pWD->prn), hwnd, LB_PRINTERS);
         return (MRESULT)FALSE;

    case WM_CLOSE:
         WinDismissDlg(hwnd, RC_OK);
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));

    case WM_CONTROL:
         NotifyCode  = SHORT2FROMMP(mp1);
         switch(NotifyCode)
            {
               case  LN_SELECT :
               case  LN_ENTER  :
                 {
                  ULONG i = 0;

                  i = (ULONG)WinSendDlgItemMsg(hwnd,
                                               LB_PRINTERS,
                                               LM_QUERYSELECTION,
                                               (MPARAM)0,  (MPARAM)0);
                  if ( i == LIT_NONE )
                     {
                       WinEnableWindow(WinWindowFromID(hwnd, DID_SETUP), FALSE);
                       WinEnableWindow(WinWindowFromID(hwnd, DID_FONTNAME), FALSE);
                       break;
                     } else {
                       WinEnableWindow(WinWindowFromID(hwnd, DID_SETUP), TRUE);
                       WinEnableWindow(WinWindowFromID(hwnd, DID_FONTNAME), TRUE);
                     }

                  i = (ULONG)WinSendDlgItemMsg(hwnd,
                                        LB_PRINTERS,
                                        LM_QUERYITEMHANDLE,
                                        MPFROMLONG(i), (MPARAM)0);
                   pWD->prn.iQueue = i;
                  }
                break;

              default:
                    return( (MRESULT)WinDefDlgProc( hwnd, msg, mp1, mp2));
            }
         break;

    case WM_COMMAND:
         switch(SHORT1FROMMP(mp1))
           {
            case DID_FONTNAME:
                  fnProcessFontDlg( hwnd, &(pWD->prn));
                 break;

            case DID_CANCEL:
                  WinDismissDlg(hwnd, -1 );
                 break;

            case DID_SETUP:
                  fnPrnQueryJobProperties( &(pWD->prn) );
                 break;

            case DID_OK:
             {
             ULONG i = 0;

             i = (ULONG)WinSendDlgItemMsg(hwnd,
                                  LB_PRINTERS,
                                  LM_QUERYSELECTION,
                                  (MPARAM)0,  (MPARAM)0
                                 );

             if ( i != LIT_NONE )
                {
                i = (ULONG)WinSendDlgItemMsg(hwnd,
                                      LB_PRINTERS,
                                      LM_QUERYITEMHANDLE,
                                      (MPARAM)i, (MPARAM)0);
                 pWD->prn.iQueue = i;
                }
             }
             WinDismissDlg(hwnd, RC_OK);

             return (MRESULT)FALSE;

            default:
                return(WinDefDlgProc(hwnd, msg, mp1, mp2));
           }
         break;

    default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
 return (MRESULT)0;
} /* End of PrintDlg */

/* Ŀ
 *  Create Printer thread                                           
 *  - Returns ThreadID or -1 for error                              
 * 
*/
TID  APIENTRY fnCreatePrinterObject(PHWND phwndNotify)
{
   TID       tidThread;

   tidThread = _beginthread( fnPrinterObjectThread,
                             0,
                             SYSMEM_STACKSIZE,
                             (PVOID)phwndNotify
                           );

 return tidThread;
}

/********************************************************/
/*     Printer Object Window Thread                    */
/********************************************************/
VOID   _Optlink   fnPrinterObjectThread(PVOID phwndNotify)
{
 HAB   hab;
 HMQ   hmq;
 QMSG  qmsg;
 PHWND  phwndTemp = phwndNotify;
 HWND  hwndTemp = *phwndTemp;
 HWND  hwndPrinter;
 CHAR  *szObjectClass = "CV32_GPI_PRINT";

 /* Set this thread two notches below interface thread */
 DosSetPrty(PRTYS_THREAD, PRTYC_NOCHANGE, -10, (TID)0);


 if(hab = WinInitialize(0))                /* get process anchor block */
  {
 if(hmq = WinCreateMsgQueue(hab,100))      /* create required msgq     */
   {
 if(WinRegisterClass(hab,szObjectClass,wpPrinter,0L, sizeof(ULONG) ))
    {
 if(hwndPrinter = WinCreateWindow( HWND_OBJECT, szObjectClass, 0, 0L,
              0,0,0,0, 0, HWND_TOP, IDO_PRINTER, (PHWND)&hwndTemp, 0))
     {

/****************************************************************************/
    /** Message loop - getmsg and dispatch to winprocs while not WC_QUIT **/
/****************************************************************************/
 while(WinGetMsg(hab, &qmsg, 0, 0, 0))
                  WinDispatchMsg(hab,&qmsg) ;


     } /* Create Window */
     WinDestroyWindow(hwndPrinter) ;

     /********************************/
     /********************************/
     /*    Kill Interface thread     */
     /********************************/
     /********************************/
     WinPostMsg(hwndTemp, UM_INFORM, MPFROMSHORT(IDO_PRINTER), 0L);
     /********************************/

    } /* RegisterWindow */
    WinDestroyMsgQueue(hmq);

   } /* CreateQueue */
   WinTerminate(hab) ;

  } /* WinInit */
 DosEnterCritSec(); // normal thread exit.
 _endthread();

}                                          /*** End of Printer Thread ***/

/*Ŀ
 * Printer Object WndProc                                              
 *
*/
MRESULT EXPENTRY wpPrinter( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
 BOOL        brc      = 0;
 HWND   hwndClient    = (HWND)0;
 PPRTREQ     pRQ      = (PPRTREQ)0;

 hwndClient = (HWND)WinQueryWindowULong(hwnd, QWL_USER);

 switch(msg)
   {
    case WM_CREATE:    /* mp1 = ptr.hwndClient          */
      if(mp1)
       {
         PHWND phwnd = (PHWND)mp1;
         hwndClient = *phwnd;
         WinSetWindowULong(hwnd, QWL_USER, (ULONG)hwndClient);

         WinPostMsg(hwndClient, UM_INFORM,        /* Send our HWND to Client */
                    MPFROMSHORT(IDO_PRINTER), MPFROMHWND(hwnd));
       }
      return(WinDefWindowProc(hwnd, msg, mp1, mp2));

    case UM_INFORM:                     /* mp1.1=function, mp2=parm */
      switch(SHORT1FROMMP(mp1))
       {
        case IDF_CLOSE:                      /* Shutdown Processing */
          WinPostMsg(hwnd, WM_QUIT, 0L, 0L); /* --- see object.main */
          break;

        case IDF_PRINT:                     /* Start Printing       */
           {
            pRQ = PVOIDFROMMP(mp2);

            pRQ->pquei.hAB = WinQueryAnchorBlock( hwnd);
            brc = fnPrintPage( pRQ );
            WinPostMsg(hwndClient, UM_INFORM,
                      MPFROMSHORT(IDF_PRINT), MPFROMSHORT(brc));
           }
          break;

        default:
          return (MRESULT)0;
       }  /* end while UM_INFORM */
       break;

    default:
         return(WinDefWindowProc(hwnd, msg, mp1, mp2));
   }
  return (MRESULT)0;
} /* End of Printer Object */

 /* Go through the queue list and add each of the   */
 /* queue entries to the list box from which the    */
 /* user can select a printer to use and/or         */
 /* configure                                       */
LONG   APIENTRY fnPrintListBox(PPRN pprn, HWND hwnd, USHORT idListBox )
{
 register INT i = 0, n = 0, z = 0;          /* Loop Counters           */


 for ( i = 0; i < pprn->cQueues; i++ )
  {
    /* Insert item and store index */
   n = (INT)LONGFROMMR( WinSendDlgItemMsg(hwnd,
                                          idListBox,
                                          LM_INSERTITEM,
                                          MPFROMSHORT(LIT_END),
                                          MPFROMP(pprn->pquei[i].szDesc)
                                         ) );

     WinSendDlgItemMsg(hwnd,
                       idListBox,
                       LM_SETITEMHANDLE,
                       (MPARAM)n,
                       (MPARAM)i
                      );

   /* If the default printer, save the name to set */
   /* selection   */
  if (  i == pprn->iQueue )  z = n;

  } // end of for loop

   WinSendDlgItemMsg(hwnd,
                     idListBox,
                     LM_SELECTITEM,
                     MPFROMSHORT(z),
                     MPFROMSHORT(TRUE)
                    );

 WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwnd,idListBox));

 return (z);
 } // end of print queue block



/* --- PrnCreatePrinterList ---------------------------- [ Public ] ---   */
/*     This function is   construct a list of printers.       */
/*     Upon Entry:                    */
/*     PPRN pprn; = Printer Control Structure Pointer         */
/*     pprn.iQueue= set to application default queue index    */
/*     Upon Exit:                     */
/*     PrnCreatePrinterList =  TRUE : Queue List Retrieved    */
/*             = FALSE : No Queue List Retrieved     */
/* --------------------------------------------------------------------   */

BOOL APIENTRY fnPrnCreatePrinterList(PPRN pprn)
{
 PCHAR     pch;            /* String Character Pointer  */
 PPRQINFO3 pPRQInf3;     /* Queue Information         */
 ULONG     aulVersion[2];  /* Version Number Holder     */
 ULONG     cReturned;      /* Returned Queue Count      */
 ULONG     cTotal;         /* Total Count               */
 ULONG     cbNeeded;       /* Buffer Size               */
 register INT i;           /* Loop Counter              */
 register UINT n;          /* Loop Counter              */

 DosQuerySysInfo( QSV_VERSION_MAJOR,
                  QSV_VERSION_MINOR,
                  aulVersion,
                  (3UL * sizeof(ULONG))
                );

 /* Determine the amount of memory required for */
 /* queue   information          */

 SplEnumQueue((PSZ)NULL,   3UL, (PVOID)NULL, 0UL, &cReturned, &cTotal,
              &cbNeeded,   NULL);

            /* Check   to see if any printers were detected   */
            /* and if none were found, return with the no  */
            /* no queue list retrieved flag       */
if ( cTotal == 0L )
   {
   pprn->cQueues = 0;
   return(FALSE);
   }
            /* Allocate memory for the queue   list     */

if ( DosAllocMem((PPVOID)(PVOID)&pPRQInf3, cbNeeded, PAG_READ  | PAG_WRITE | PAG_COMMIT) )
   {
   pprn->cQueues = 0;
   return(FALSE);
   }
            /* Retrieve the queue list from the system  */

SplEnumQueue((PSZ)NULL,   3UL, (PVOID)pPRQInf3, cbNeeded, &cReturned,
       &cTotal, &cbNeeded, NULL);

pprn->pquei = (PQUEINFO)malloc(sizeof(QUEINFO) * cTotal);

            /* Scan the queue list looking for the entry   */
            /* that contains the default queue and when */
            /* found, save the queue  index entry    */

for ( i = 0; i < cReturned; i++ )
{
   if ( pPRQInf3[i].pszName )
       {
       pprn->pquei[i].pszName = (PSZ)malloc((n = strlen(pPRQInf3[i].pszName) + 1));
       memcpy(pprn->pquei[i].pszName, pPRQInf3[i].pszName, n);
       }
   else
       pprn->pquei[i].pszName = NULL;


   if ( pPRQInf3[i].pszComment )
       memcpy(pprn->pquei[i].szDesc, pPRQInf3[i].pszComment, 48);
   else
       strcpy(pprn->pquei[i].szDesc, pprn->pquei[i].pszName );


   if ( pPRQInf3[i].pszDriverName )
       {
       pprn->pquei[i].pszDriverName = (PSZ)malloc((n = strlen(pPRQInf3[i].pszDriverName) + 1));
       memcpy(pprn->pquei[i].pszDriverName, pPRQInf3[i].pszDriverName, n);
       }
   else
       pprn->pquei[i].pszDriverName = NULL;

   if ( pPRQInf3[i].pszPrinters )
       {
       pprn->pquei[i].pszPrinters = (PSZ)malloc((n = strlen(pPRQInf3[i].pszPrinters) + 1));
       memcpy(pprn->pquei[i].pszPrinters, pPRQInf3[i].pszPrinters, n);
       }
   else
       pprn->pquei[i].pszPrinters = NULL;

   pprn->pquei[i].pDriverData = (PSZ)malloc((ULONG)pPRQInf3[i].pDriverData->cb);
   memcpy(pprn->pquei[i].pDriverData, pPRQInf3[i].pDriverData, (ULONG)pPRQInf3[i].pDriverData->cb);

            /* Check   for the  default  queue       */

   if ( ((aulVersion[0] == 20UL) && (aulVersion[1] >= 10UL)) ||
        ((aulVersion[0] >  20UL) && (aulVersion[1] >= 10UL)) )
       if ( pPRQInf3[i].fsType & PRQ3_TYPE_APPDEFAULT )
       {
            pprn->iQueue   = i;
           strcpy(pprn->szQueue, pPRQInf3[i].pszName);
       }

}   /* end for loop */

if ( (aulVersion[0] == 20UL) && (aulVersion[1] < 10UL) )
{
            /* Get the default queue  to use         */

   if ( PrfQueryProfileString(HINI_PROFILE, "PM_SPOOLER", "QUEUE", NULL,
              pprn->szQueue, 256UL) )
       {
            /* Truncate the semicolon delimiter in the queue  */
            /* name                */

       pch = strchr(pprn->szQueue, ';');
       *pch = 0;
       }
   else
            /* No default queue name  found, use the first */
            /* printer from the list           */

       strcpy(pprn->szQueue, pPRQInf3->pszName);

            /* Scan the queue list looking for the entry   */
            /* that contains the default queue and when */
            /* found, save the queue  index entry    */

   for ( i = 0;  i < cReturned; i++ )
       if ( !strcmp(pprn->szQueue, pPRQInf3[i].pszName) )
     {
     pprn->iQueue   = i;
     break;
     }
} /* end if old style */

            /* Save the queue information within the printer  */
            /* information structure  and save the number of  */
            /* queues found within the system as well and  */
            /* return the queues found flag       */

pprn->cQueues =  (INT)cReturned;

DosFreeMem((PVOID)pPRQInf3);

return(TRUE);
}

/* --- PrnDestroyPrinterList --------------------------- [ Public ] ---   */
/*                                                                        */
/*     This function is   used to  destroy  a constructed printer list.   */
/*     Upon Entry:                                                        */
/*     PPRN pprn; = Printer Control Structure Pointer                     */
/*                                                                        */
/*     Upon Exit:                                                         */
/*     PrnDestroyPrinterList =  TRUE : List Destroyed                     */
/*                           = FALSE : Error                              */
/*                                                                        */
/* --------------------------------------------------------------------   */

BOOL APIENTRY fnPrnDestroyPrinterList(PPRN pprn)

{
register INT i;           /* Loop Counter         */

            /* Check   to make  sure that memory was allocated   */
            /* for the list and if the case,   release  it */
if ( pprn != (PPRN)0 )
   {
   for ( i = 0;  i < pprn->cQueues; i++ )
       {
       if ( pprn->pquei[i].pszName )
     free(pprn->pquei[i].pszName);

       if ( pprn->pquei[i].pszDriverName )
     free(pprn->pquei[i].pszDriverName);

       if ( pprn->pquei[i].pszPrinters )
     free(pprn->pquei[i].pszPrinters);

       free(pprn->pquei[i].pDriverData);
       }

   free (pprn->pquei ); /* release the array allocated by the create routine */
   pprn->cQueues = 0;
   return(TRUE);
   }
            /* No memory allocated for the list, return */
            /* error   flag              */
return(FALSE);
}


/* --- PrnOpenDC --------------------------------------- [ Public ] ---
 *
 *     This function is used to open a device context on a specified
 *     printer.
 *
 *     Upon Entry:
 *     PQUEINFO pquei;   = The selected print struc
 *     PSZ pszComment    = Job comment text
 *
 *     Upon Exit:
 *     HDC               = Printer Queued Device Context
 * --------------------------------------------------------------------
*/
HDC APIENTRY fnPrnOpenDC(PQUEINFO pquei, PSZ pszComment)
{
 CHAR achDriverName[256];   /* Driver Name           */
 DEVOPENSTRUC dop;          /* Device Open Structure */
 register INT i;            /* Loop Counter          */

 if ( pquei == (PQUEINFO)0 )
   return((HDC)NULL);

/* Initialize the driver  data structure    */
 memset(&dop, 0,  sizeof(DEVOPENSTRUC));

/* Set the logical address         */
 dop.pszLogAddress = pquei->pszName;

/* Set the driver name (ie. PSCRIPT)     */
 i = (INT)strcspn(pquei->pszDriverName,  ".");
 if( i != 0 )
 {
   dop.pszDriverName = memcpy(achDriverName, pquei->pszDriverName, (UINT)i);
   achDriverName[i] = 0;
 } else dop.pszDriverName = strcpy(achDriverName, pquei->pszDriverName);

/* set ptr to print driver data structure */
 dop.pdriv  = (PVOID)pquei->pDriverData;

/* Set data type (ie. PM_Q_STD or PM_Q_RAW) */
 dop.pszDataType  = "PM_Q_STD";
 dop.pszComment   =  pszComment; // Comment for OD_Q

/* Other data values include:                                    */
/*  dop.pszQueueProcName   = NULL;  // queue processor;          */
/*                                  //   NULL =>  use default    */
/*  dop.pszQueueProcParams = NULL;  // parms for queue processor */
/*  dop.pszSpoolerParams   = NULL;  // spooler parms (use NULL!) */
/*  dop.pszNetworkParams   = NULL;  // network parms (use NULL!) */

 return( DevOpenDC(pquei->hAB,
                   OD_QUEUED, "*", 9L,
                   (PDEVOPENDATA)(PVOID)&dop,
                   (HDC)0L)
                  );
}

/* --- PrnOpenInfoDC ----------------------------------- [ Public ] ---
 *
 *     This function is   used to  open a device context on a specified
 *     printer.   The type of device context opened is an information
 *     type where no printing is to occur.
 *
 *     Upon Entry:
 *     PQUEINFO pQue;  = Printer Control Structure Pointer
 *
 *     Upon Exit:
 *     HDC              = Printer Info Device Context
 *
 * --------------------------------------------------------------------
*/
HDC APIENTRY fnPrnOpenInfoDC(PQUEINFO pQue)
{
CHAR achDriverName[256];   /* Driver Name           */
DEVOPENSTRUC dop;          /* Device Open Structure */
register INT i;            /* Loop Counter          */

if ( pQue == (PQUEINFO)0 )
   return((HDC)NULL);

/* Initialize the driver  data structure    */
memset(&dop, 0,  sizeof(DEVOPENSTRUC));

/* Set the logical address         */
dop.pszLogAddress = pQue->pszName;

/* Set the driver name (ie. PSCRIPT)     */
i = (INT)strcspn(pQue->pszDriverName,  ".");
if ( i != 0 )
   {
   dop.pszDriverName = memcpy(achDriverName, pQue->pszDriverName, (UINT)i);
   achDriverName[i] = 0;
   } else dop.pszDriverName = strcpy(achDriverName, pQue->pszDriverName);

/* set ptr to print driver data structure */
dop.pdriv  = (PVOID)pQue->pDriverData;

/* Set data type (ie. PM_Q_STD or PM_Q_RAW) */
dop.pszDataType  = "PM_Q_STD";

return( DevOpenDC(pQue->hAB,
                  OD_INFO, "*", 9L,
                  (PDEVOPENDATA)(PVOID)&dop,
                  (HDC)0L)
                 );
}

/* --- PrnQueryJobProperties --------------------------- [ Public ] ---
 *
 *     This function is used to get printer job properties.  It is
 *     valid for a printer to require no job properties.  In this
 *     case, *pcb == 0.
 *
 *     Upon Entry:
 *     PPRN pprn;   = Printer Control Structure   Pointer
 *     INT  iQueue; = Selected Queue Index
 *
 *     Upon Exit:
 *     PrnQueryJobProperties =  TRUE : Properties Retrieved
 *              = FALSE : Buffer to Small
 * --------------------------------------------------------------------
*/
BOOL APIENTRY fnPrnQueryJobProperties(PPRN pprn)

{
CHAR   achDeviceName[256];      /* Device Name       */
CHAR   achDriverName[256];      /* Driver Name       */
PCHAR     pch;                  /* Character Pointer */
INT    iQueue = 0;
register INT i;                 /* Index          */

if ( !pprn->cQueues )
   return(FALSE);

/* Find the driver/device delimiter and parse  */
/* out the driver name and the device name     */
iQueue = pprn->iQueue;

i = (INT)strcspn(pprn->pquei[iQueue].pszDriverName, ".");
if ( i != 0 )
   {
   /* Set the driver name          */
   memcpy(achDriverName, pprn->pquei[iQueue].pszDriverName, (UINT)i);
   achDriverName[i] = 0;

   /* Set the device name          */
   strcpy(achDeviceName, &pprn->pquei[iQueue].pszDriverName[i +  1]);
   }
else
   {
   strcpy(achDriverName, pprn->pquei[iQueue].pszDriverName);
   achDeviceName[0] = 0;
   }

/* terminate properly the printer name  */
pch = strchr(pprn->pquei[iQueue].pszPrinters, ',');
if ( pch != NULL )
   *pch = 0;

return((BOOL)(DevPostDeviceModes(pprn->hAB,
                                (PVOID)pprn->pquei[iQueue].pDriverData,
                                achDriverName,
                                achDeviceName,
                                pprn->pquei[iQueue].pszPrinters,
                                DPDM_POSTJOBPROP) == DEV_OK) );
}


/* --- fnPrnQueryPrinterName ------------------------ [ Public ] ---
 *
 *     This function is   used to  get information   on a printer.
 *
 *     Upon Entry:
 *     PPRN pprn; = Printer Control Structure Pointer
 *
 *     Upon Exit:
 *     PrnQueryPrinterName = Address of  Printer  Name
 *
 * --------------------------------------------------------------------
*/

PSZ APIENTRY fnPrnQueryPrinterName(PPRN pprn)

{

if ( !pprn )
   return((PSZ)NULL);

if ( !pprn->cQueues )
   return((PSZ)NULL);

/* Return the address of  the printer name  */
return(pprn->pquei[pprn->iQueue].pszName);
}

/* --- fnGetNextLCID ------------------------------------ [ Public ] ---
 * This function is used to find the next available lcid in the current PS
 *     Upon Entry:
 *     HPS hps;     = Presentation Space Handle
 *
 *     Upon Exit:
 *     LONG lcid; = Logical Font ID   or GPI_ERROR
 * --------------------------------------------------------------------
*/
#define MAXSETID  255
LONG APIENTRY fnGetNextLCID(HPS hps)
{
  INT       i   = 0;
  LONG  lLcid   = GPI_ERROR;
  LONG  lCount  = 0;
  BOOL  fRet    = FALSE;
  PLONG alLcids = NULL;      /* lcid currently in use */
  PLONG alTypes;             /* Font or Bitmap Flag   */
  PSTR8 aNames;              /* 8 byte arroy of logical font names */


  /************************************************/
  /* See if any local set IDs have been used yet. */
  /************************************************/
  lCount = GpiQueryNumberSetIds(hps);
  if(lCount==0)
     return(1L);

  if(lCount == GPI_ALTERROR)
     return lLcid;   // Error

  /***********************************/
  /* Find first unused local set ID. */
  /***********************************/

  alLcids = malloc( (sizeof(LONG) * lCount) );
  alTypes = malloc( (sizeof(LONG) * lCount) );
  aNames  = malloc( (sizeof(STR8) * lCount) );

  if(alLcids != NULL)
     fRet = GpiQuerySetIds(hps,lCount,alTypes,aNames,alLcids);

  if(fRet)
  {
     for( lLcid=1; lLcid<(MAXSETID+1); lLcid++)
     {
       for(i=0;( i < lCount) && (alLcids[i] != lLcid);i++);

       if(i == lCount)
          break;
     }

     if(lLcid == MAXSETID+1)
        lLcid=GPI_ERROR;
   }

 free(alLcids);
 free(alTypes);
 free(aNames);

 return(lLcid);
}

/* --- fnProcessFont()--------------------------------- [ Public ] ---
 * This function is used present the standard Font DlgBox
 *     Upon Entry:
 *     PPRN pprn;   = Pointer to Printer List Array
 *
 *     Upon Exit:
 *     FATTRS pfAttrs       = Selected Fonts Attributes
 *     SIZEF  sizefxCharBox = Outlien Fonts GpiSetCharBox() values
 *     returncode  = DID_OK or DID_CANCEL
 * --------------------------------------------------------------------
*/
LONG   APIENTRY  fnProcessFontDlg( HWND hwndOwner, PPRN pprn)
{
 HWND    hwndFontDlg;             /* Handle font dialog, or True return code */
 CHAR    szfamilyName[FACESIZE];  /* Font family name array.                 */

 HDC     hdcInfo = (HDC)0;        /* handle of print info device context     */
 HPS     hps   = (HPS)NULL;       /* handle to printer ps                    */
 SIZEL   sizl;                    /* Size of presentation page.              */
 PQUEINFO pQue  = (PQUEINFO)0;
 FONTDLG fntDialog;               /* Font dialog data structure.             */

 if ( pprn == (PPRN)0 )
    return (LONG)DID_CANCEL;


 pQue = &(pprn->pquei[pprn->iQueue]);

 hdcInfo = fnPrnOpenInfoDC( pQue );

 sizl.cx = sizl.cy = 0L;
 hps =   GpiCreatePS(pQue->hAB,
                     hdcInfo,
                     &sizl,
                     PU_TWIPS | GPIF_DEFAULT | GPIT_NORMAL | GPIA_ASSOC
                    );

 memset(&fntDialog, 0, sizeof(FONTDLG) );

 if(pQue->fFontInit)
 {
   strcpy(szfamilyName, pQue->fAttrs.szFacename);
   fntDialog.fxPointSize = pQue->fxPointSize;
 } else {
         memset(&(pQue->fAttrs),0,sizeof(FATTRS));
         szfamilyName[0]          = (CHAR)0;
         fntDialog.fxPointSize    = MAKEFIXED(10,0);
         fntDialog.fAttrs.usCodePage = (USHORT)(ULONG)GpiQueryCp( hps );
        }

 pQue->fFontInit = FALSE;

 fntDialog.cbSize         = sizeof(FONTDLG);
 fntDialog.fl             = FNTS_CENTER;
 fntDialog.clrFore        = CLR_BLACK;
 fntDialog.clrBack        = SYSCLR_WINDOW;
 fntDialog.hpsScreen      = NULLHANDLE;
 fntDialog.hpsPrinter     = hps;
 fntDialog.usWeight       = 5;
 fntDialog.usWidth        = 5;
 fntDialog.pszFamilyname  = szfamilyName;
 fntDialog.usFamilyBufLen = FACESIZE;

 hwndFontDlg = WinFontDlg(HWND_DESKTOP,
                          hwndOwner,
                          &fntDialog
                         );

 if( fntDialog.lReturn == DID_OK )
   {
    memcpy(&(pQue->fAttrs),&fntDialog.fAttrs,sizeof(FATTRS));
    pQue->sizefxCharBox.cx  = fntDialog.fxPointSize * 20L;  // convert to decipoints
    pQue->sizefxCharBox.cy  = fntDialog.fxPointSize * 20L;
    pQue->sNominalPointSize = fntDialog.fxPointSize / 0x10000;  // DISPLAY VALUE
    pQue->fxPointSize       = fntDialog.fxPointSize;
    pQue->lEmHeight         = fntDialog.lEmHeight;
    pQue->fFontInit = TRUE;
   }

 GpiAssociate(hps, (HDC)0 );
 GpiDestroyPS(hps);
 DevCloseDC(hdcInfo);

 return fntDialog.lReturn;
}


/* --- fnCreateFont( )--------------------------------- [ Public ] ---
 * This function is used create a logical font and set its charbox
 *     Upon Entry:
 *     HPS hPS;       = Presentation Space Handle
 *     FATTRS pfattrs = Pointer to Selected Font (from ProcessFontDlg()
 *     SIZEEF pSizefxCharBox = sizef containing desired point size(240)
 *                                                               = 12
 *     Upon Exit:
 *     LONG   lLCid = Current Font ID
 *            rc    = or GPI_ERROR
 * --------------------------------------------------------------------
*/
LONG  APIENTRY  fnCreateFont(HPS hps, PFATTRS pfAttrs, PSIZEF psizefxCharBox )
{
 LONG lLcid = 0L;
 LONG lrc   = 0L;

 lLcid = fnGetNextLCID(hps);


 if(pfAttrs->fsFontUse == FATTR_FONTUSE_OUTLINE)
    {
     pfAttrs->lMatch          = 0L;
     pfAttrs->lMaxBaselineExt = 0;
     pfAttrs->lAveCharWidth   = 0;
    } else pfAttrs->lMatch          = 0L;

 /* rc= FONT_MATCH, FONT_DEFAULT, GPI_ERROR  */
 lrc = GpiCreateLogFont(hps,
                        NULL,
                        lLcid,
                        pfAttrs
                       );

 if (lrc == GPI_ERROR)
    return lrc;

 GpiSetCharSet(hps,lLcid);  // rc = true false
 if(pfAttrs->fsFontUse == FATTR_FONTUSE_OUTLINE)
 {
   /**************************/
   /* Set the character box. */
   /**************************/
   GpiSetCharBox(hps, psizefxCharBox);
 }

 return lLcid;
}


/* *************************************************************
 * fnPrintRequest() Routine to submit a print page request to
 *                  printer object window.
 * -Input  : pWD        Program window data structure
 * -Input  : pszMessage One line buffer to text to print
 * -OutPut : Message posted to PrinterObjectWindow
 * -Returns: undefined - api return code from data fetches
 * *************************************************************
*/
APIRET   APIENTRY fnPrintRequest( PWINDOWINFO pWD, PSZ pszMessage)
{
  PPRTREQ      pRQ = (PPRTREQ)0;
  PQUEINFO      pQ = (PQUEINFO)0;
  PDRIVDATA pDrive = (PDRIVDATA)0;
  APIRET    api_rc = 0;
  register       i = 0;

  if(pWD == (PWINDOWINFO)0)
    return -1;

  pRQ = (PPRTREQ)malloc( sizeof(PRTREQ) );
  if(pRQ == (PPRTREQ)0)
    return -1;

/* Get copy of Queue Info   */
  memcpy( (PVOID)&(pRQ->pquei.pszName), (PVOID)&(pWD->prn.pquei[pWD->prn.iQueue].pszName), sizeof(QUEINFO) );
  pDrive = (PDRIVDATA)pWD->prn.pquei[pWD->prn.iQueue].pDriverData;
  pRQ->pquei.pDriverData = malloc( pDrive->cb + 256);
  memcpy( (PVOID)pRQ->pquei.pDriverData,
          (PVOID)pDrive,
          (LONG)pDrive->cb
        );

/* Get real data and set api_rc *********************************************/
/* Format the request structure */
  pRQ->lRows         = 1L;
  pRQ->lCols         = 1L;

/* Load Titles    */
  strcpy(pRQ->szHeader, "CV2TAB Pkg: Gprint Library Header");
  strcpy(pRQ->szFooter, "CV2TAB Pkg: Gprint Library Footer");
  strcpy(pRQ->szTitle, pWD->szTitle );

  i = strlen(pszMessage);   /* this is not a good way to tell buffer lengths */
  if(i > sizeof(pWD->szTitle) ) i = sizeof(pWD->szTitle) - 1;

  pRQ->pszPrintBuffer = (PSZ)malloc( i+128 );
  memset(pRQ->pszPrintBuffer, 0, i+128 );
  memcpy(pRQ->pszPrintBuffer, pszMessage, i );

  api_rc = 0;                                      /* ok to procede */
/* Get real data and set api_rc *********************************************/

/* Send request to Print thread */
  if (api_rc == 0)
    if(pWD->hwndPrtObject)
     WinPostMsg(pWD->hwndPrtObject, UM_INFORM, MPFROMSHORT(IDF_PRINT), MPFROMP(pRQ) );
     else {
           WinMessageBox( HWND_DESKTOP, HWND_DESKTOP,
                         "Current Print Job Request Failed.",
                         "  Internal error: PrinterObjectWindow Invalid.",
                         0, MB_APPLMODAL | MB_CUACRITICAL | MB_CANCEL
                         );

           api_rc = -1;      /* no prt obj wnd indicates major prog error */
           free(pRQ->pquei.pDriverData);
           free( pRQ->pszPrintBuffer );                  /* Clean Up */
           free( pRQ );                                  /* Clean Up */
           WinAlarm(HWND_DESKTOP, WA_ERROR);
           WinAlarm(HWND_DESKTOP, WA_ERROR);
          }

 return api_rc;
}

/* Ŀ  */
/*    Print the screen using the prepared request buffer.        */
/*   */
ULONG   APIENTRY  fnPrintPage(PPRTREQ pRQ)
{
 ULONG       rc = 0;
 UCHAR       szLineBuffer[TEXT_BUFFER];
 UCHAR       szTitleLine[TEXT_BUFFER];
 HDC         hdcPrinter;
 HPS         hpsPrinter;
 FONTMETRICS fmPrn;                         /* Font Metrics Info          */
 HCINFO      hci;                           /* Hard Copy Information Structure */
 LONG        lcid, lLineInc;                /* Logical Font ID            */
 LONG        lCapsVRes = 0;
 LONG        lCapsHRes = 0;
 POINTL      ptl;                           /* Display Point              */
 SIZEL       sizlPage;                      /* Page Size   Holder         */
 HAB         hab;
 APIRET      api_rc;
 USHORT      usJobID;                       /* Spooler Job ID       */
 LONG        lOut;                          /* Output Count         */
 LONG        lLoop;                         /* Misc Loop Counter    */
 SIZEF       sizefFont;
 PQUEINFO    pQue = (PQUEINFO)0;
 register    y = 0, z = 0;
 register    x = 0;


 if(!pRQ)
     return TRUE;    /* Error Trap */

 hab = pRQ->pquei.hAB;
 pQue = &(pRQ->pquei);

 memset(szLineBuffer, 0, sizeof(szLineBuffer));

 hdcPrinter = fnPrnOpenDC( pQue, pRQ->szTitle);

 sizlPage.cx = sizlPage.cy = 0L;
 hpsPrinter =   GpiCreatePS(hab, hdcPrinter, &sizlPage, PU_TWIPS |
                            GPIF_DEFAULT | GPIT_NORMAL | GPIA_ASSOC);


   /* Determine the hard copy capabilities for the   */
   /* form selected. Note that all values returned   */
   /* are in millimetres except for  the xPels and   */
   /* yPels                    elements.             */
   /*                                                */
   /* Coordinate Conversions                         */
   /*     TWIPS     where TWIPS are 1440 per inch    */
   /*   --------                                     */
   /*   mm  * 57                                     */
   /*                                                */
   /*    millimeters (mm)                            */
   /*   -------------                                */
   /*   25.4 x inches                                */
   /*                                                */
   /*    World.Coordinates _for GpiSetCharBox()      */
   /*   ------------------------------------         */
   /*  (Dev.Pels * Desired.Points) / 72.2818         */
   /*  (  45     *       12      ) / 72.2818         */
   /* in DevQueryCaps( CAPS_GRAPHICS_CHAR_WIDTH      */
   /*   or           ( CAPS_GRAPHICS_CHAR_HEIGHT     */
   /*   or           ( CAPS_HORIZONTAL_FONT_RES      */
   /*   or           ( CAPS_VERTICAL_FONT_RES        */
   /* then GpiSetCharBox(answer.x, answer.y)         */
   /* ********************************************** */

 lOut = DevQueryHardcopyCaps(hdcPrinter,  0L, 0L, NULL);
 for ( lLoop = 0; lLoop < lOut; lLoop++)
       {
        DevQueryHardcopyCaps(hdcPrinter,  lLoop, 1L,  &hci);

        if ( hci.flAttributes && HCAPS_CURRENT )
         break;
       }


             /* Start the printing of   the selected file by */
             /* indicating the start of the document within   */
             /* device context                    */
 api_rc = DevEscape( hdcPrinter,
                     DEVESC_STARTDOC,
                     (LONG)strlen(pRQ->szTitle),
                     pRQ->szTitle,
                     NULL, NULL);

 if ( api_rc == DEVESC_ERROR )
    {
     GpiAssociate(hpsPrinter,(HDC)NULL);
     GpiDestroyPS(hpsPrinter);
     DevCloseDC(hdcPrinter);
    }


  /* Get the new font metrics for the default font  */
  /* and determine the number of lines that can be  */
  /* contained within the page, set font to 10pt    */

  DevQueryCaps(hdcPrinter, CAPS_VERTICAL_FONT_RES, 1L, &lCapsVRes);
  DevQueryCaps(hdcPrinter, CAPS_HORIZONTAL_FONT_RES, 1L, &lCapsHRes);

 lcid = GPI_ERROR;
 if(pQue->fFontInit)
    lcid = fnCreateFont( hpsPrinter,
                         &(pQue->fAttrs),
                         &(pQue->sizefxCharBox)
                       );

 GpiQueryFontMetrics(hpsPrinter, sizeof(FONTMETRICS), &fmPrn);

 if( ( fmPrn.fsDefn & FM_DEFN_OUTLINE ) && (lcid == GPI_ERROR ) )
   {
    sizefFont.cx = sizefFont.cy = MAKEFIXED(200,0);    // 10pt
    GpiSetCharBox(hpsPrinter, &sizefFont);
    GpiQueryFontMetrics(hpsPrinter, sizeof(FONTMETRICS), &fmPrn);
   }


 /* fixup some page control vars */
 lLineInc = fmPrn.lEmHeight;
 ptl.x = hci.xLeftClip * 57L;   // normally xleft is 5   (10 is recco)
 ptl.y = (hci.yTopClip - hci.yBottomClip )  * 57L;   // 57 * value(mm) equals Twips

 /* Process Header String */
    ptl.y -= (lLineInc *  1L);              /* printpos setting on top of charbox before this move down 1 */
    strcpy(szTitleLine, "TopLine : " );
    strcat(szTitleLine, pRQ->szHeader );
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szTitleLine), szTitleLine);

    ptl.y -= lLineInc;
    strcpy(szTitleLine, "NextLine: " );
    strcat(szTitleLine, pRQ->szTitle );
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szTitleLine), szTitleLine);

    ptl.y -= lLineInc;
    strcpy(szTitleLine, "NextLine: " );
    strcat(szTitleLine, pRQ->szPrinter );
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szTitleLine), szTitleLine);

 /* Process Body of Page */
    ptl.y -= ( lLineInc * 2L );
    strcpy(szLineBuffer, pRQ->pszPrintBuffer);  // Limited to 512 inc zero
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "fm.Facename = %s", fmPrn.szFacename);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "Q.sNominalPointSize = %d", pQue->sNominalPointSize);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "Logical Char id   = %d", lcid);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "fm.lEmHeight=%ld,  fm.lEmInc=%ld, Q.lEmHeight=%ld",
                           fmPrn.lEmHeight, fmPrn.lEmInc, pQue->lEmHeight);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "fm.sNominalPointSize = %d, fm.sMin = %d, fm.sMax = %d",
            fmPrn.sNominalPointSize, fmPrn.sMinimumPointSize, fmPrn.sMaximumPointSize);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "fm.sXDeviceRes=%d,  fm.sYDeviceRes=%d",
                          fmPrn.sXDeviceRes, fmPrn.sYDeviceRes);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "Dev.CapsVfontRes=%ld, dev.CapsHfontRes=%ld",
                          lCapsVRes, lCapsHRes);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    sprintf(szLineBuffer, "prg.lLineInc=%ld same as fmPrn.lEmHeight", lLineInc );
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    strcpy(szTitleLine, "hci.szFormname=%s, ");
    strcat(szTitleLine, " hci.cx=%ld,");
    strcat(szTitleLine, " hci.cy=%ld,");
    strcat(szTitleLine, " hci.xPels=%ld,");
    strcat(szTitleLine, " hci.yPels=%ld");
    sprintf(szLineBuffer, szTitleLine,
            hci.szFormname, hci.cx, hci.cy, hci.xPels, hci.yPels);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    strcpy(szTitleLine, "hci.xLeftClip=%ld,");
    strcat(szTitleLine, " hci.yBottomClip=%ld,");
    strcat(szTitleLine, " hci.xRightClip=%ld,");
    strcat(szTitleLine, " hci.yTopClip=%ld");
    sprintf(szLineBuffer, szTitleLine,
                     hci.xLeftClip, hci.yBottomClip, hci.xRightClip, hci.yTopClip);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);

    ptl.y -= lLineInc;
    strcpy(szTitleLine, "hps.sizl.cx=%ld,");
    strcat(szTitleLine, " hps.Sizl.cy=%ld");
    sprintf(szLineBuffer, szTitleLine, sizlPage.cx, sizlPage.cy);
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szLineBuffer), szLineBuffer);


 /* Process Footer String */
    ptl.y = hci.yBottomClip  * 57L;
    strcpy(szTitleLine, pRQ->szFooter );
    GpiCharStringAt(hpsPrinter, &ptl, strlen(szTitleLine), szTitleLine);


 DevEscape(hdcPrinter, DEVESC_ENDDOC, 0L,   NULL, &lOut, (PBYTE)&usJobID);

 GpiDeleteSetId(hpsPrinter, lcid);
 GpiAssociate(hpsPrinter, (HDC)NULL);
 GpiDestroyPS(hpsPrinter);
 DevCloseDC(hdcPrinter);

 free( pRQ->pquei.pDriverData);
 free( pRQ->pszPrintBuffer );                  /* Clean Up */
 free( pRQ);
 return rc;
}
