/*
 * GPI-Based Printing.  This file was created by James Scott Jr.
 * USDate 28 Feb 1997.
*/


/*
 *----------------------*
 *  Constant definitions
 *----------------------*
*/
#define WIN_CLASS_NAME     "WC_GPRINT"   /* Client window class name         */
#define BUFFERSIZE                 60    /* Maximum buffer length            */
#define THREAD_STACK_SIZE       65535    /* Stack size for secondary threads */

#define MSG_TITLE                  64
#define EMSG_TITLE                128
#define TEXT_BUFFER               257
#define FNAME_LEN                 256
#define RC_OK                       0
#define SYSMEM_STACKSIZE        65535    /* Segment Size for STACKS           */

#define ALLOC_PUBLIC                0    /* Allocate public memory            */
#define ALLOC_PRIVATE     sizeof(PVOID)  /* Allocate private memory           */
#define ALLOC_SYSTEM (2 * sizeof(PVOID)) /* Allocate system memory            */
#define ALLOC_LAST                  3    /* Number of mem. pointers in appl.  */
#define WINDOW_WORDS_SZ (8 * sizeof(PVOID)) // reserve 8 window words

/*--------------------------*/
/* Identifier constants     */
/*--------------------------*/

/* Window IDs */
#define IDW_FRAME_WINDOW            101

#define IDO_PRINTER                 103             // Object Window

/* Menu item IDs */
#define IDM_FILE                    1001
#define IDM_FILE_SETUP              1002
#define IDM_FILE_PRINT              1003
#define IDM_FILE_ABOUT              1004
#define IDM_POPUP                   1100

/* Dialog and dialog controls IDs */
#define IDD_ABOUT                   201

#define IDD_PRINT                   210
#define   LB_PRINTERS               211
#define   DID_SETUP                 212
#define   DID_FONTNAME              213


/* Message ID's */
#define UM_BASE                     WM_USER+1000
#define UM_INFORM                   UM_BASE+1 /* mp1.1=function, mp2=parm    */
#define    IDF_CLOSE                100       /* Shutdown Processing Function*/
#define    IDF_PRINT                101       /* Start Printing  bStatus.ctrl*/


/*------------------*/
/* Type definitions */
/*------------------*/

/*
 *Ŀ
 *             Print Request Data                                             
 *
*/
typedef  struct _QUEINFO {             /* Individual Print Queue Details         */
   PSZ          pszName;               /* Device Name                            */
   PSZ          pszDriverName;         /* Driver Name                            */
   PSZ          pszPrinters;           /* Printers                               */
   UCHAR        szDesc[64];            /* Queue Description                      */
   HAB          hAB;                   /* Anchor block for list                  */
   SIZEF        sizefxCharBox;         /* Selected Outline font charbox settings */
   FIXED        fxPointSize;           /* size of user selected font             */
   LONG         lEmHeight;             /* effective pointsize                    */
   SHORT        sNominalPointSize;     /* Selected Image Fonts Point Size        */
   BOOL         fFontInit;             /* is FATTRS Initialized ?                */
   FATTRS       fAttrs;                /* Font attributes data structure.        */
   PVOID        pDriverData;           /* Device Data                            */
   } QUEINFO ;                         /* quei */

typedef  QUEINFO  *PQUEINFO;           /* pquei */

typedef  struct _PRN {                 /* Master Print List          */
   CHAR     szQueue[256];              /* Default Queue Name         */
   INT      iQueue;                    /* Selected Printer Index     */
   INT      cQueues;                   /* Total Queue Count          */
   PQUEINFO pquei;                     /* Array of Queue Information */
   HAB      hAB;                       /* Anchor block for list      */
   } PRN ;                             /* printer list control       */

typedef  PRN *PPRN;                    /* pprn */

typedef struct _PRTREQ {               /* Onetime Print Request Struc */
  UCHAR   szPrinter[256];              /* Printer Name                */
  UCHAR   szHeader[64];                /* Page header                 */
  UCHAR   szFooter[64];                /* Page footer                 */
  UCHAR   szTitle[256];                /* Body/Doc title              */
  LONG    lRows;                       /* Number of lines in buffer   */
  LONG    lCols;                       /* Number of columns in buffer */
  ULONG   lPrintBufferSize;            /* True size of the buffer     */
  PSZ     pszPrintBuffer;              /* Print Text buffer           */
  QUEINFO pquei;                       /* Individual print q data     */
 } PRTREQ;
typedef PRTREQ  *PPRTREQ;

/*
 *Ŀ
 *         END Print Request Data                                             
 *
*/

/* Window instance data */
typedef struct
   {
    HWND   hwnd;                      /* handle of main client window           */
    HWND   hwndMenu;                  /* handle of popup menu                   */
    HAB    hab;                       /* anchor block of main client window     */
    HWND   hwndPrtObject;             /* handle to printer thread object window */
    UCHAR  szTitle[256];              /* window title                           */
    ULONG  lPrintBufferSize;          /* True size of the print buffer          */
    PRN    prn;                       /* print dialog control structure         */
   } WINDOWINFO, *PWINDOWINFO;



/*---------------------*/
/* Function Prototypes */
/*---------------------*/
int              main();
MRESULT APIENTRY wpClientWndProc(HWND, ULONG, MPARAM, MPARAM);

/****************************************************************************/
#define DB_RAISED       0x0400
#define DB_DEPRESSED    0x0800
LONG     APIENTRY fnProcessAboutDlg(HWND hwndFrame, PSZ pszAppName);
MRESULT  EXPENTRY wpAboutProc (HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
/****************************************************************************/
LONG     APIENTRY fnProcessFontDlg( HWND hwndOwner, PPRN pprn);
LONG     APIENTRY fnCreateFont(HPS hps, PFATTRS pfAttrs, PSIZEF psizefxCharBox );
LONG     APIENTRY fnGetNextLCID(HPS hps);
/****************************************************************************/
LONG     APIENTRY fnProcessPrintDlg(HWND hwndFrame);
MRESULT  EXPENTRY wpPrintDlg( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
LONG     APIENTRY fnPrintListBox(PPRN pprn, HWND hwnd, USHORT idListBox );
/****************************************************************************/
TID      APIENTRY fnCreatePrinterObject(PHWND phwndNotify);
VOID     _Optlink fnPrinterObjectThread(PVOID phwndNotify);
MRESULT  EXPENTRY wpPrinter( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
ULONG    APIENTRY fnPrintPage(PPRTREQ pRQ);
APIRET   APIENTRY fnPrintRequest( PWINDOWINFO pWD, PSZ pszMessage);
/****************************************************************************/
BOOL     APIENTRY fnPrnCreatePrinterList(PPRN pprn);
BOOL     APIENTRY fnPrnDestroyPrinterList(PPRN pprn);
HDC      APIENTRY fnPrnOpenDC(PQUEINFO pquei, PSZ pszComment);
HDC      APIENTRY fnPrnOpenInfoDC( PQUEINFO pQue );
BOOL     APIENTRY fnPrnQueryJobProperties(PPRN pprn);
PSZ      APIENTRY fnPrnQueryPrinterName(PPRN pprn);
/****************************************************************************/
