/* ****************************************************************************

   Program Name
         CV2TABPM.EXE      Sample PM Program to illustrate Control Functions

   Program Function
         Displays a set of dialog controls on a regular pm client window.
         includes routines to support tabbing and changing the view.

   File Contents
       One standard Presentation Manager Main routine, with;
       - View_A: Standard dialog control windows
       - View_B: Standard dialog control windows


   Author: James Scott
         : Fort Wayne, Indiana
         : 219-460-4213 e-mail jscott@us.ibm.com
   ****************************************************************************
   Change Log.

          Created:  06-06-1998
   ****************************************************************************
*/

/*Ŀ*/
/* Include files                                                            */
/**/
#define INCL_BASE
#define INCL_GPI
#define INCL_WIN                        /* relevant parts of               */
                                        /* the PM header file              */
#include <os2.h>                        /* PM header file                  */

/*Ŀ*/
/* Include files                                                            */
/**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "CV2TAB.H"          /*Support Routines */

/*Ŀ*/
/* Include files      Application Private                                   */
/**/
#include "CV2TABPM.H"

/* Ŀ*/
/*  Global Variables                                                         */
/* */
/* None */

INT fnCreate100View(HWND hwnd);

/* Ŀ*/
/*  Main Procedure...                                                       */
/* */
int main( )
{
 HAB        hab            = (HAB)0;
 HWND       hwndFrame      = (HWND)0;
 HWND       hwndClient     = (HWND)0;
 HMQ        hmq            = (HMQ)0;
 QMSG       qmsg;
 FRAMECDATA fcdata;
 CHAR       szTitle[60];             /* Title string                       */
 SCROLLINFO SclInfo;

 hab = WinInitialize(0);

 hmq=WinCreateMsgQueue(hab,0);

 strcpy( szTitle, "Client Controls Library");

 WinRegisterClass(hab, "CV2_TAB", wpWndProc, CS_SYNCPAINT | CS_SIZEREDRAW, sizeof(ULONG) );

 fcdata.cb            = sizeof(FRAMECDATA);   /* Specify size.             */
 fcdata.flCreateFlags = FCF_TITLEBAR | FCF_SYSMENU    | FCF_MENU       |
                      FCF_MINMAX     | FCF_ICON       | FCF_ACCELTABLE |
                      FCF_AUTOICON   | FCF_SIZEBORDER | FCF_TASKLIST   |
                      FCF_VERTSCROLL | FCF_HORZSCROLL ;

 fcdata.hmodResources = (HMODULE)0;         /* Resource file not in DLL. */
 fcdata.idResources   = IDW_MAIN;           /* Resource file identifier. */

 hwndFrame = WinCreateWindow(HWND_DESKTOP, WC_FRAME, szTitle,
                             0L, 0, 0, 0, 0, (HWND)0, HWND_TOP,
                             IDW_MAIN, &fcdata, NULL);

 hwndClient = WinCreateWindow(hwndFrame, "CV2_TAB", (PSZ)NULL,
                              0L, 0,0,0,0, (HWND)0, HWND_TOP,
                              FID_CLIENT, &SclInfo, NULL);

/*Ŀ*/
/* Message dispatching loop  */
/**/
 while ( WinGetMsg(hab,(PQMSG)&qmsg, (HWND)0,0,0))
         WinDispatchMsg( hab, (PQMSG)&qmsg );


 WinDestroyWindow(hwndFrame);
 WinDestroyMsgQueue( hmq );
 WinTerminate( hab );

 return (0);
}

/*Ŀ */
/* Main Window Procedure                                                */
/* */
MRESULT EXPENTRY wpWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
 PSCROLLINFO pScl = (PSCROLLINFO)WinQueryWindowULong(hwnd, QWL_USER);

 switch( msg )
 {
  case WM_COMMAND:
       return fnClientCommand(hwnd,msg,mp1,mp2, pScl);

  case WM_CHAR:
       return fnHandleWMChar(hwnd,msg,mp1,mp2);

  case WM_HSCROLL:
       return fnHandleHScroll(hwnd, mp2, pScl);

  case WM_VSCROLL:
       return fnHandleVScroll(hwnd, mp2, pScl);

  case WM_SIZE:
       return fnHandleWMSize(hwnd, mp1, mp2, pScl);


  case WM_PAINT:
       fnDefaultPaint(hwnd, SYSCLR_DIALOGBACKGROUND );
       return(WinDefWindowProc(hwnd,msg,mp1,mp2));

  case WM_CREATE:
       WinSetWindowULong(hwnd, QWL_USER, (ULONG)mp1);

       WinPostMsg(hwnd, UM_CREATE, 0L, 0L);
       return WinDefWindowProc(hwnd,msg,mp1,mp2);

  case UM_CREATE:
       fnSetScrollType(hwnd, BITSCROLL, pScl);
       fnCreateFullView(hwnd);
       fnGetWindowList(hwnd, pScl);
       fnCenterView(hwnd, pScl);
       WinInvalidateRect(hwnd, NULL, TRUE);
       break;

  default:
       return(WinDefWindowProc(hwnd,msg,mp1,mp2));
 } /* end switch */

 return(FALSE);
} /* End wpWndProc */

/* Ŀ*/
/*  ClientCommand() - WM_COMMAND Processing for the client window       */
/*  Returns : mresult                                                   */
/* */
MRESULT    fnClientCommand(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2, PSCROLLINFO pScl)
{

 if (pScl == (PSCROLLINFO)0)
    return (MRESULT)0;

 switch(SHORT1FROMMP( mp1 ))
 {
  case MI_VIEW_A:
       fnClearView(hwnd, pScl);                       // remove current
       fnCreateFullView(hwnd);                  // create this one
       fnGetWindowList(hwnd, pScl);
       WinInvalidateRect(hwnd, NULL, TRUE);
       break;

  case MI_VIEW_B:
       fnClearView(hwnd, pScl);                       // remove current
       fnCreate100View(hwnd);             // create this one
       fnGetWindowList(hwnd, pScl);
       WinInvalidateRect(hwnd, NULL, TRUE);
       break;

  case MI_REFRESH:
       WinInvalidateRect(hwnd, NULL, TRUE);
       break;

  case MI_EXIT:
       WinPostMsg(hwnd, WM_CLOSE, 0L, 0L);
       break;

     default:
         return(WinDefWindowProc(hwnd,msg,mp1,mp2));
 }   /* end switch */

 return (MRESULT)0;
}   /* end ClientCommand */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
INT      fnCreateFullView(HWND hwnd)
{
 HWND hTarget;
 POINTL  aptl[2];
 BOOL    rc;

 aptl[0].x =   7L;
 aptl[0].y =   60L;
 aptl[1].x =   74L;
 aptl[1].y =   70L;
 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC, "Radio Buttons",
      SS_GROUPBOX |WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
      aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
      hwnd, HWND_BOTTOM,  257, 0L, NULL);
 }

 aptl[0].x =   13L;
 aptl[0].y =   108L;
 aptl[1].x =   56L;
 aptl[1].y =   10L;
 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON, "Choice ~1",
     BS_AUTORADIOBUTTON | WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
     aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
     hwnd, HWND_BOTTOM,  258, 0L, NULL);
 }

 aptl[0].x =   13L;
 aptl[0].y =   95L;
 aptl[1].x =   56L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Choice ~2",
     BS_AUTORADIOBUTTON | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  259, 0L, NULL);

 aptl[0].x =   13L;
 aptl[0].y =   81L;
 aptl[1].x =   56L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Choice ~3",
     BS_AUTORADIOBUTTON | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  260, 0L, NULL);

 aptl[0].x =   13L;
 aptl[0].y =   67L;
 aptl[1].x =   56L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Choice ~4",
     BS_AUTORADIOBUTTON | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  261, 0L, NULL);

 aptl[0].x =   87L;
 aptl[0].y =   60L;
 aptl[1].x =   74L;
 aptl[1].y =   70L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_STATIC, "Check Boxes",
     SS_GROUPBOX |WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  262, 0L, NULL);

 aptl[0].x =   93L;
 aptl[0].y =   108L;
 aptl[1].x =   62L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Selection ~A",
     BS_AUTOCHECKBOX |WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  263, 0L, NULL);

 aptl[0].x =   93L;
 aptl[0].y =   95L;
 aptl[1].x =   62L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Selection ~B",
     BS_AUTOCHECKBOX | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  264, 0L, NULL);

 aptl[0].x =   93L;
 aptl[0].y =   81L;
 aptl[1].x =   62L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Selection ~C",
     BS_AUTOCHECKBOX | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  265, 0L, NULL);

 aptl[0].x =   93L;
 aptl[0].y =   67L;
 aptl[1].x =   62L;
 aptl[1].y =   10L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "Selection ~D",
     BS_AUTOCHECKBOX | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  266, 0L, NULL);

 aptl[0].x =   19L;
 aptl[0].y =   135L;
 aptl[1].x =   139L;
 aptl[1].y =   19L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 hTarget = WinCreateWindow(hwnd,   WC_STATIC, "Text",
     SS_HALFTONERECT |WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  267, 0L, NULL);

    fnSetWindowPresColor(hTarget,PP_FOREGROUNDCOLOR, RGB_BLACK);

 aptl[0].x =   11L;
 aptl[0].y =   140L;
 aptl[1].x =   140L;
 aptl[1].y =   20L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 hTarget = WinCreateWindow(hwnd,   WC_STATIC, "Text",
     SS_HALFTONERECT |WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  268, 0L, NULL);

    fnSetWindowPresColor(hTarget,PP_FOREGROUNDCOLOR, RGB_BLUE);

 aptl[0].x =   7L;
 aptl[0].y =   33L;
 aptl[1].x =   153L;
 aptl[1].y =   24L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_LISTBOX, "",
     WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  269, 0L, NULL);

 aptl[0].x =  43L;
 aptl[0].y =  142L;
 aptl[1].x =  22L;
 aptl[1].y =  16L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,  WC_STATIC, "#2000",
     SS_ICON | WS_GROUP |WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM, 270, 0L, NULL);

 aptl[0].x =   65L;
 aptl[0].y =   144L;
 aptl[1].x =   72L;
 aptl[1].y =   12L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 hTarget = WinCreateWindow(hwnd,   WC_STATIC, "Client View/2",
     SS_TEXT |DT_LEFT | DT_VCENTER | WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  271, 0L, NULL);

    fnSetWindowPresFont(hTarget,PP_FONTNAMESIZE, (PSZ)"14.Tms Rmn");
    fnSetWindowPresColor(hTarget,PP_FOREGROUNDCOLOR, RGB_WHITE);
    fnSetWindowPresColor(hTarget,PP_BACKGROUNDCOLOR, RGB_BLUE);

 aptl[0].x =   68L;
 aptl[0].y =   20L;
 aptl[1].x =   84L;
 aptl[1].y =   8L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd, WC_ENTRYFIELD, "EF",
     ES_LEFT | ES_AUTOSCROLL |ES_MARGIN | WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  272, 0L, NULL);

 aptl[0].x =   21L;
 aptl[0].y =   20L;
 aptl[1].x =   41L;
 aptl[1].y =   8L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_STATIC, "Entryfield",
     SS_TEXT |DT_LEFT | DT_TOP | WS_GROUP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  273, 0L, NULL);

 aptl[0].x =   11L;
 aptl[0].y =   2L;
 aptl[1].x =   40L;
 aptl[1].y =   13L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "~View B",
     BS_PUSHBUTTON |BS_DEFAULT | WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM, MI_VIEW_B, 0L, NULL);

 aptl[0].x =   60L;
 aptl[0].y =   2L;
 aptl[1].x =   41L;
 aptl[1].y =   13L;
 if(WinMapDlgPoints (hwnd, aptl, 2, TRUE))
 WinCreateWindow(hwnd,   WC_BUTTON, "~Regular",
     BS_PUSHBUTTON |WS_TABSTOP | WS_SYNCPAINT | WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  275, 0L, NULL);

 WinSendDlgItemMsg(hwnd, 258, BM_SETCHECK, (MPARAM)1, 0L);

 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 2" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 3" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 4" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 5" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 6" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 7" ));
 WinSendDlgItemMsg(hwnd,269, LM_INSERTITEM,
                   MPFROM2SHORT(LIT_END, 0 ),
                   MPFROMP("Selection Line 8" ));

 WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwnd, 258) );

 return 19 ;
}

/* ***************************************************************** */
INT  fnCreate100View(HWND hwnd)
{
 POINTL  aptl[2];
 BOOL rc = FALSE;

 aptl[0].x =   14L;
 aptl[0].y =   119L;
 aptl[1].x =   55L;
 aptl[1].y =   14L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,      "~Add New",
     BS_PUSHBUTTON| WS_TABSTOP| WS_VISIBLE |  WS_GROUP,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_NEW, 0L, NULL);
 }

 aptl[0].x =   14L;
 aptl[0].y =   105L;
 aptl[1].x =   55L;
 aptl[1].y =   14L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,      "~Change",
     BS_PUSHBUTTON| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_CHANGE, 0L, NULL);
 }

 aptl[0].x =   14L;
 aptl[0].y =   91L;
 aptl[1].x =   55L;
 aptl[1].y =   14L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,      "~Delete",
     BS_PUSHBUTTON| WS_VISIBLE | WS_DISABLED,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_DELETE, 0L, NULL);
 }

 aptl[0].x =   14L;
 aptl[0].y =   77L;
 aptl[1].x =   55L;
 aptl[1].y =   14L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,   "~Next",
     BS_DEFAULT| BS_PUSHBUTTON | WS_VISIBLE ,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  MI_VIEW_A, 0L, NULL);
 }

 aptl[0].x =   72L;
 aptl[0].y =   76L;
 aptl[1].x =   260L;
 aptl[1].y =   61L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC,        "Employee Information",
     SS_GROUPBOX| WS_TABSTOP| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  101, 0L, NULL);
 }

 aptl[0].x =   78L;
 aptl[0].y =   118L;
 aptl[1].x =   26L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC,           "Name",
     SS_TEXT| DT_LEFT| WS_GROUP| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  107, 0L, NULL);
 }

 aptl[0].x =   124L;
 aptl[0].y =   118L;
 aptl[1].x =   138L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_ENTRYFIELD,      "",
     WS_TABSTOP | ES_LEFT | WS_VISIBLE |  ES_MARGIN | WS_GROUP,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_NAME, 0L, NULL);
 }

 aptl[0].x =   78L;
 aptl[0].y =   104L;
 aptl[1].x =   28L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC,           "Phone",
     SS_TEXT| DT_LEFT| WS_GROUP| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  109, 0L, NULL);
 }

 aptl[0].x =   124L;
 aptl[0].y =   104L;
 aptl[1].x =   75L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_ENTRYFIELD,      "",
     WS_TABSTOP | ES_LEFT | WS_VISIBLE |  ES_RIGHT | ES_MARGIN,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_PHONE, 0L, NULL);
 }

 aptl[0].x =   77L;
 aptl[0].y =   90L;
 aptl[1].x =   42L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC,           "E-Mail ID",
     SS_TEXT| DT_LEFT| WS_GROUP| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  111, 0L, NULL);
 }

 aptl[0].x =   124L;
 aptl[0].y =   90L;
 aptl[1].x =   124L;
 aptl[1].y =   8L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_ENTRYFIELD,      "",
     WS_TABSTOP | ES_LEFT | WS_VISIBLE |  ES_MARGIN,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_EMAIL, 0L, NULL);
 }

 aptl[0].x =   273L;
 aptl[0].y =   81L;
 aptl[1].x =   54L;
 aptl[1].y =   47L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_STATIC,        "Status",
     SS_GROUPBOX| WS_TABSTOP| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  113, 0L, NULL);
 }

 aptl[0].x =   277L;
 aptl[0].y =   109L;
 aptl[1].x =   40L;
 aptl[1].y =   10L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,    "Ac~tive",
     BS_AUTOCHECKBOX| WS_TABSTOP| WS_VISIBLE |  WS_GROUP,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_ACTIVE, 0L, NULL);
 }

 aptl[0].x =   277L;
 aptl[0].y =   99L;
 aptl[1].x =   46L;
 aptl[1].y =   10L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,    "~Retired",
     BS_AUTOCHECKBOX| WS_VISIBLE ,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_RETIRED, 0L, NULL);
 }

 aptl[0].x =   277L;
 aptl[0].y =   89L;
 aptl[1].x =   40L;
 aptl[1].y =   10L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_BUTTON,    "~Temp",
     BS_AUTOCHECKBOX| WS_VISIBLE,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_TEMP, 0L, NULL);
 }

 aptl[0].x =   14L;
 aptl[0].y =   7L;
 aptl[1].x =   327L;
 aptl[1].y =   67L;

 rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);
 if( rc )
 {
  WinCreateWindow(hwnd,   WC_MLE,             "Readme.Txt",
     MLS_WORDWRAP | MLS_HSCROLL | MLS_VSCROLL | WS_VISIBLE |  MLS_HSCROLL |MLS_VSCROLL | WS_GROUP,
    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,
    hwnd, HWND_BOTTOM,  IDC_MLE, 0L, NULL);
 }

 WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwnd, IDC_NEW) );
 return 16 ;
}

/* ***************************************************************** */
