/* ************************************************************************ */
/* DialogTemplate to C CreateViewProc                                       */
/*  Date: 8-17-1992                                                         */
/*  Date: 6-05-1998   UpDated output format                                 */
/*  Program Name: CV2DtoW.Cmd  Dialog to WinCreateWindow exec.              */
/*                                                                          */
/*  Description :                                                           */
/*    The purpose of this exec is to convert the source output files from   */
/*    the DLGBOX.EXE/DLGEDIT dialog box editor into C procedures that issue */
/*    WinCreateWindow() calls to create views use on a client window.  The  */
/*    WinCreateWindow() calls must be prefixed with the WinMapDlgPoints()   */
/*    call in order to convert the dialog units of the source file into PELS*/
/*    used by the WinCreateWindow() call.                                   */
/*                                                                          */
/* CAUTION: The quoted text of controls must end with - DoublequoteComma -  */
/*          this is normally how the dlgbox editor writes them.             */
/*                                                                          */
/*          CTLDATA & PRESPARAMS are preserved a comments in output file.   */
/*                                                                          */
/*          The 'NOT' operator used by the resource compiler is not         */
/*          supported the C Compiler.  You will have to edit each occurance */
/*          to the C not operator "~" manually.                             */
/* ************************************************************************ */
/* input format expected                                                    */
/* =====================                                                    */
/* ...                                                                      */
/* DLGTEMPLATE  DlgConstantName                        == C Procedure Name  */
/* BEGIN                                                                    */
/*      DIALOG ...                                                          */
/*      BEGIN                                                               */
/*           control spec's  (DLGEDIT or DLGBOX formats)                    */
/*           ...                                                            */
/*      END                                                                 */
/* END       ...                                                            */
/*    Repeated as many times as desired                                     */
/* ************************************************************************ */

call DrainQueue

PARSE UPPER ARG fnSource
do
 /* Get filenames - if source is null prompt for it */
 if fnSource == "" | length(fnSource) < 2 then
                       do
                        say "Syntax: CV2DtoW Source.dlg"
                        say "Please enter dialog source filename.dlg"
                        PARSE PULL fnSource
                        if fnSource == "" then return 1
                       end

 /* compose output filename.[V|H] */
 do
   x = pos('.',fnSource,1)
   if x == 0 then x = min((length(fnSource)+1), 9)
   x = x - 1
   if x > 0 then fnTarget = left(fnSource,x)||'.V'
   if x > 0 then fnHeader = left(fnSource,x)||'.VH'
   '@if exist 'fnHeader 'erase 'fnHeader
   '@if exist 'fnTarget 'erase 'fnTarget
 end

end

do
  sepdata =  "/* ***************************************************************** */"
  say sepdata
  say fnTarget '-- C file contains the Code to create the views.'
  say fnHeader '-- H file contains the declarations for the above.'
 /* line seperator constant */
 call on NOTREADY name NotFoundExit
end

/* ***************************************************** */
/* Main  routine - find a dlgtemplate in the source file */
/* ***************************************************** */
/* 1. Find Top of Template         NewControl() = 6      */
/* 2. Find body Template  - second NewControl() = 5      */
/* 3. Process Template      while  NewControl() = 1      */
/* 4. Write the C File                                   */
/* ***************************************************** */
do until lines(fnSource) == 0
   ldata = linein(fnSource)

   if length(ldata) == 0 then iterate           /* skip blank lines */
     else controlline = strip(ldata)


   if ( NewControl() <> 6 ) then iterate        /* %%%%%% step 1 %%%%%%%%% */


   say sepdata
   say 'Proc Name: ' word(controlline, 2)
   ProcName = word(controlline, 2)              /* Get Name of output proc */


   Counter = 0
   do until Counter == 2                        /* %%%%%% step 2 %%%%%%%%% */
      ldata = linein(fnSource)
      if ( NewControl() == 5 ) then Counter = Counter + 1
   end

                                                /* %%%%%% step 3 %%%%%%%%% */
   do until ( lines(fnSource) == 0) | ( NewControl() == 4)

      ldata = linein(fnSource)

      if length(ldata) == 0 then iterate

      do while ( NewControl() <> 4 )

         if ( NewControl() <> 1 ) then
             do
               ldata = linein(fnSource)
               iterate
             end

          ControlWord = word(strip(ldata),1)

          select
             when ControlWord == "CONTROL" then
             call CONTROL

             when ControlWord == "ENTRYFIELD" then
             call ENTRYFIELD

             when ControlWord == "MLE" then
             call MLE

             when ControlWord == "COMBOBOX" then
             call COMBOBOX

             when ControlWord == "LISTBOX" then
             call LISTBOX

             when ControlWord == "LTEXT" then
             call LTEXT

             when ControlWord == "RTEXT" then
             call RTEXT

             when ControlWord == "CTEXT" then
             call CTEXT

             when ControlWord == "GROUPBOX" then
             call GROUPBOX

             when ControlWord == "ICON" then
             call ICON

             when ControlWord == "PUSHBUTTON" then
             call PUSHBUTTON

             when ControlWord == "RADIOBUTTON" then
             call RADIOBUTTON

             when ControlWord == "DEFPUSHBUTTON" then
             call DEFPUSHBUTTON

             when ControlWord == "AUTORADIOBUTTON" then
             call AUTORADIOBUTTON

             when ControlWord == "CHECKBOX" then
             call CHECKBOX

             when ControlWord == "AUTOCHECKBOX" then
             call AUTOCHECKBOX

             when ControlWord == "CTLDATA" then
             call SPECIAL_COMMENT

             when ControlWord == "PRESPARAMS" then
             call SPECIAL_COMMENT

          otherwise
          say "Error(" ControlWord ") Un-identified Control Type"
          ldata = linein(fnSource)
          iterate
          end  /* select */

      end  /* process a template */

   counter =  CreateOutputFile()
   end /* build a proc */

end /* process a dlg file */
return 0

/* ***************************************************************** */
/* Procedure Name: NewControl                                        */
/* Determines if the current line of data is the start of a new      */
/* control statement.                                                */
/* return 0 if current data line is not the start                    */
/* return 1 if current data line is the start of a supported ctrl    */
/* return 2 if current data line is the start of a unsupported ctrl  */
/* return 3 if current data line is the start of a New Proc body     */
/* return 4 if current data line is a END statement                  */
/* return 5 if current data line is a BEGIN statement                */
/* return 6 if current data line is the start of a New Template      */
/* ***************************************************************** */
NewControl: procedure expose ldata
do
  ControlWord = word(strip(ldata),1)
  select
     when ControlWord == "DLGTEMPLATE"     then return 6
     when ControlWord == "WINDOWTEMPLATE"  then return 6

     when ControlWord == "BEGIN"           then return 5
     when ControlWord == "{"               then return 5

     when ControlWord == "END"             then return 4
     when ControlWord == "}"               then return 4

     when ControlWord == "DIALOG"          then return 3
     when ControlWord == "FRAME"           then return 3

     when ControlWord == "CTLDATA"         then return 1
     when ControlWord == "PRESPARAMS"      then return 1

     when ControlWord == "WINDOW"          then return 1
     when ControlWord == "CONTROL"         then return 1
     when ControlWord == "ENTRYFIELD"      then return 1
     when ControlWord == "MLE"             then return 1
     when ControlWord == "COMBOBOX"        then return 1
     when ControlWord == "LISTBOX"         then return 1
     when ControlWord == "LTEXT"           then return 1
     when ControlWord == "RTEXT"           then return 1
     when ControlWord == "CTEXT"           then return 1
     when ControlWord == "GROUPBOX"        then return 1
     when ControlWord == "ICON"            then return 1
     when ControlWord == "PUSHBUTTON"      then return 1
     when ControlWord == "RADIOBUTTON"     then return 1
     when ControlWord == "DEFPUSHBUTTON"   then return 1
     when ControlWord == "AUTORADIOBUTTON" then return 1
     when ControlWord == "CHECKBOX"        then return 1
     when ControlWord == "AUTOCHECKBOX"    then return 1
     otherwise
     return 0
  end
end
return 0

/* ***************************************************************** */
/* Procedure Name: MakeLine                                          */
/* Rebuild the multiple line control statement into one line.        */
/* ***************************************************************** */
MakeLine: procedure expose ldata fnSource
 TextLine = ""
do until ( NewControl() > 0 )
             TextLine = TextLine||strip(ldata)
             ldata = linein(fnSource)
end
return TextLine

/* ***************************************************************** */
/*  %%%%%%%%%% Control Formatter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/* TypeA  CONTROL title,  id, x, y, cx, cy, class, style  -- CONTROL */
/*  -or-  CONTROL number, id, x, y, cx, cy, class, style  -- BITMAPS */
/*           X     A     B   C  D  E   F   G      H                  */
/* TypeB  controltype title, id, x, y, cx, cy, style     -- OTHERS   */
/*           X        A      B   C  D  E   F   H                     */
/* TypeC  LISTBOX id, x, y, cx, cy, style                -- LISTBOX  */
/*           X     B  C  D  E   F   H                                */
/* TypeD  ICON    number, id, x, y, cx, cy, style        -- ICON     */
/*           X    A       B   C  D  E   F   H                        */
/* ***************************************************************** */
/* NOTES:                                                            */
/* TypeD(A) must be a numeric value - else you edit it in the C file */
/* all(H) styles - operator 'NOT' is not support by C, edit to '~'   */
/* ***************************************************************** */
MakeTypeA: procedure expose MsgLine
do
   /* Check for Bitmaps - They do not have doublequoted string for a title */
   /* their titles are numeric                                             */
   PARSE VAR MsgLine X ' ' A ',' .
   WhatType = strip(A)
   if( WhatType > 0 )
   then do
          PARSE VAR MsgLine X ' ' A ',' B ',' C ',' D ',' E ',' F ',' G ',' H
          if (H == "") then H = "NULL"
          MsgLine = X||' "#'||strip(A)||'",'||B||','||C||','||D||','||E||','||F||', '||G||', '||H
        end
   else do
          PARSE VAR MsgLine X ' ' A '",' B ',' C ',' D ',' E ',' F ',' G ',' H
          if (H == "") then H = "NULL"
          MsgLine = X||' '||A||'",'||B||','||C||','||D||','||E||','||F||', '||G||', '||H
        end
end
return MsgLine

MakeTypeB: procedure expose MsgLine WinClass WinStyle
do
   PARSE VAR MsgLine X ' ' A '",' B ',' C ',' D ',' E ',' F ',' H
   if (H == "")
      then H = WinStyle
      else H = WinStyle||' | '||H
   MsgLine = X||' '||A||'",'||B||','||C||','||D||','||E||','||F||', '||WinClass||', '||H
end
return MsgLine

MakeTypeC: procedure expose MsgLine WinClass WinStyle
do
   PARSE VAR MsgLine X ' ' B ',' C ',' D ',' E ',' F ',' H
   if (H == "")
      then H = WinStyle
      else H = WinStyle||' | '||H
   MsgLine = X||'         "", '||strip(B)||','||C||','||D||','||E||','||F||', '||WinClass||', '||H
end
return MsgLine

MakeTypeD: procedure expose MsgLine WinClass WinStyle
do
   PARSE VAR MsgLine X ' ' A ',' B ',' C ',' D ',' E ',' F ',' H
   if (H == "")
      then H = WinStyle
      else H = WinStyle||' | '||H
   MsgLine = X||'            "#'||strip(A)||'",'||B||','||C||','||D||','||E||','||F||', '||WinClass||', '||H
end
return MsgLine

/* ***************************************************************** */
/*  %%%%%%%%%% Control Parsers %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/* Rebuild the multiple line control statement into one line.        */
/* Format text line into a CONTROL(TypeA) format.                    */
/* Pushes onto the QUEUE 1 Line that is the entire CONTROL statement */
/* ***************************************************************** */
SPECIAL_COMMENT: procedure expose ldata fnSource
          do
             MsgLine = MakeLine()
             QUEUE MsgLine
          end
         return 0

CONTROL: procedure expose ldata fnSource
          do
             MsgLine = MakeLine()
             MsgLine = MakeTypeA()
             QUEUE MsgLine
          end
         return 0

ENTRYFIELD: procedure  expose ldata fnSource
          do
             WinClass = "WC_ENTRYFIELD"
             WinStyle = "WS_TABSTOP | ES_LEFT | WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

MLE: procedure  expose ldata fnSource
          do
             WinClass = "WC_MLE"
             WinStyle = "MLS_WORDWRAP | MLS_HSCROLL | MLS_VSCROLL | WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

COMBOBOX: procedure  expose ldata fnSource
          do
             WinClass = "WC_COMBOBOX"
             WinStyle = "CBS_SIMPLE| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

LISTBOX: procedure  expose ldata fnSource
          do
             WinClass = "WC_LISTBOX"
             WinStyle = "WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeC()
             QUEUE MsgLine
          end
          return 0

LTEXT: procedure  expose ldata fnSource
          do
             WinClass = "WC_STATIC"
             WinStyle = "SS_TEXT| DT_LEFT| WS_GROUP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

RTEXT: procedure  expose ldata fnSource
          do
             WinClass = "WC_STATIC"
             WinStyle = "SS_TEXT| DT_RIGHT| WS_GROUP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

CTEXT: procedure  expose ldata fnSource
          do
             WinClass = "WC_STATIC"
             WinStyle = "SS_TEXT| DT_CENTER| WS_GROUP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

GROUPBOX: procedure  expose ldata fnSource
          do
             WinClass = "WC_STATIC"
             WinStyle = "SS_GROUPBOX| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

ICON: procedure  expose ldata fnSource
          do
             WinClass = "WC_STATIC"
             WinStyle = "SS_ICON| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeD()
             QUEUE MsgLine
          end
          return 0

PUSHBUTTON: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_PUSHBUTTON| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

RADIOBUTTON: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_RADIOBUTTON| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

DEFPUSHBUTTON: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_DEFAULT| BS_PUSHBUTTON| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

AUTORADIOBUTTON: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_AUTORADIOBUTTON| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

CHECKBOX: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_CHECKBOX| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

AUTOCHECKBOX: procedure  expose ldata fnSource
          do
             WinClass = "WC_BUTTON"
             WinStyle = "BS_AUTOCHECKBOX| WS_TABSTOP| WS_VISIBLE"
             MsgLine = MakeLine()
             MsgLine = MakeTypeB()
             QUEUE MsgLine
          end
          return 0

/* ***************************************************************** */
/* Procedure Name: CreateOutputFile                                  */
/* Pull controls of Queue and write the output file.                 */
/* ***************************************************************** */
CreateOutputFile: procedure expose ProcName fnTarget fnHeader sepdata

do
   count = 0
   x = lineout(fnTarget, sepdata)
   ldata = "INT  fnCreate"||ProcName||"View(HWND hwnd)"
    x = lineout(fnTarget, ldata)
    x = lineout(fnHeader, ldata||';')
   ldata = "{"
    x = lineout(fnTarget, ldata)
   ldata = " POINTL  aptl[2];"
    x = lineout(fnTarget, ldata)
   ldata = " BOOL rc = FALSE;"
    x = lineout(fnTarget, ldata)
    x = lineout(fnTarget, " ")

   do while QUEUED() > 0
       PARSE PULL iData
       PARSE VAR iData X ' ' A '",' B ',' C ',' D ',' E ',' F ',' G ',' H
       if (strip(X) == "CTLDATA") | (strip(X) == "PRESPARAMS")
         then do
                 ldata = "/* "||iData||" */"
                 x = lineout(fnTarget, ldata)
                 iterate
              end

       ldata = " aptl[0].x = " C||"L;"
        x = lineout(fnTarget, ldata)
       ldata = " aptl[0].y = " D||"L;"
        x = lineout(fnTarget, ldata)
       ldata = " aptl[1].x = " E||"L;"
        x = lineout(fnTarget, ldata)
       ldata = " aptl[1].y = " F||"L;"
        x = lineout(fnTarget, ldata)
       ldata = "  "
        x = lineout(fnTarget, ldata)

       ldata = " rc = WinMapDlgPoints (hwnd, aptl, 2, TRUE);"
        x = lineout(fnTarget, ldata)

       ldata = " if( rc )"
        x = lineout(fnTarget, ldata)

       ldata = " {"
        x = lineout(fnTarget, ldata)

       ldata = '  WinCreateWindow(hwnd, ' G||',' A||'",'
        x = lineout(fnTarget, ldata)
       ldata = "    "H||","
        x = lineout(fnTarget, ldata)
       ldata = "    aptl[0].x, aptl[0].y, aptl[1].x, aptl[1].y,"
        x = lineout(fnTarget, ldata)
       ldata = "    hwnd, HWND_BOTTOM," B||", 0L, NULL);"
        x = lineout(fnTarget, ldata)

       ldata = " }"
        x = lineout(fnTarget, ldata)

        x = lineout(fnTarget, " ")
    count = count + 1
   end
   ldata = ' return 'count ';'
    x = lineout(fnTarget, ldata)
   ldata = "}"
    x = lineout(fnTarget, ldata)
    x = lineout(fnTarget, " ")
    x = lineout(fnTarget, sepdata)
   say count "Controls written to proc: Create"||ProcName||"View() in file:" fnTarget
end
return 0

NotFoundExit:
do
  Say "System Error: Procedure Halted" CONDITION('C')
  say QUEUED() 'Lines found in Queue, PURGED.'
  do while QUEUED() \== 0
   PULL ldata
  end
  exit
end
return 1

/* Drain Queue Command */
DrainQueue:
do
  say QUEUED() 'Lines found in Queue.'

  do while QUEUED() \== 0
   PULL ldata
  end
end
return

/*
 The CONTROL statement can be used to define a window control of any class. Window classes may be user defined
 of one of a predefined set provided by the operating system. The following classes are provided in OS/2 Version 2.0.

 WC_FRAME                Application frame control.
 WC_STATIC               Text and group boxes.
 WC_BUTTON               Push button, check box or radio button.
 WC_COMBOBOX             Combination of an entry field and list box.
 WC_ENTRYFIELD           Single line entry field.
 WC_MLE                  Multiple line entry field.
 WC_LISTBOX              List box.
 WC_MENU                 Application action bar, menus and popup menus.
 WC_SCROLLBAR            Horizontal or vertical scroll bar.
 WC_TITLEBAR             Application title bar.
 WC_SPINBUTTON           Spin button entry field.
 WC_CONTAINER            Container list.
 WC_SLIDER               Horizontal or vertical slider bar.
 WC_VALUESET             Value set control.
 WC_NOTEBOOK             Notebook control.

 These controls make up the standard user interface components for applications. The following example shows a
 simple listbox control.

  CONTROL "", 1, 10, 20, 60, 40, WC_LISTBOX, WS_VISIBLE




 In addition to the general form of the CONTROL statement, there are special control statements for commonly used
 controls.  These statements define the attributes of the child control windows that appear in the window.

 Control statements have this general form:



  controltypetext,id,x,y,cx,cy

  
      ,style

             Ŀ
                                      
  BEGINDIALOG statementEND
               CONTROL statementĴ
               WINDOW statement

 The LISTBOX control statement is an exception to this general form because it does
 not have a text field.

 Type...................Class...........Style..................................................
 FRAME                  WC_FRAME        WS_VISIBLE
 ENTRYFIELD             WC_ENTRYFIELD   WS_TABSTOP, ES_LEFT, WS_VISIBLE
 COMBOBOX               WC_COMBOBOX     CBS_SIMPLE, WS_TABSTOP, WS_VISIBLE
 LISTBOX                WC_LISTBOX      WS_VISIBLE
 MLE                    WC_MLE          MLS_VSCROLL | WS_VISIBLE
 LTEXT                  WC_STATIC       SS_TEXT, DT_LEFT, WS_GROUP, WS_VISIBLE
 RTEXT                  WC_STATIC       SS_TEXT, DT_RIGHT, WS_GROUP, WS_VISIBLE
 CTEXT                  WC_STATIC       SS_TEXT, DT_CENTER, WS_GROUP, WS_VISIBLE
 GROUPBOX               WC_STATIC       SS_GROUPBOX, WS_TABSTOP, WS_VISIBLE
 ICON                   WC_STATIC       SS_ICON, WS_VISIBLE
 PUSHBUTTON             WC_BUTTON       BS_PUSHBUTTON, WS_TABSTOP, WS_VISIBLE
 RADIOBUTTON            WC_BUTTON       BS_RADIOBUTTON, WS_TABSTOP, WS_VISIBLE
 DEFPUSHBUTTON          WC_BUTTON       BS_DEFAULT, BS_PUSHBUTTON, WS_TABSTOP, WS_VISIBLE
 AUTORADIOBUTTON        WC_BUTTON       BS_AUTORADIOBUTTON, WS_TABSTOP, WS_VISIBLE
 CHECKBOX               WC_BUTTON       BS_CHECKBOX, WS_TABSTOP, WS_VISIBLE
 AUTOCHECKBOX           WC_BUTTON       BS_AUTOCHECKBOX, WS_TABSTOP, WS_VISIBLE

*/
